@@ -197,7 +197,7 @@ my @mysqld_rules=
  { '#host' => \&fix_host },
  { 'port' => \&fix_port },
  { 'socket' => \&fix_socket },
- { '#log-error' => \&fix_log_error },
+ { 'log-error' => \&fix_log_error },
  { 'general-log' => 1 },
  { 'plugin-dir' => sub { $::plugindir } },
  { 'general-log-file' => \&fix_log },
@@ -311,7 +311,7 @@ sub post_check_client_groups {
 				$first_mysqld->name());
 
  # Then generate [client.<suffix>] for each [mysqld.<suffix>]
- foreach my $mysqld ( $config->like('mysqld.') ) {
+ foreach my $mysqld ( $config->like('mysqld\.') ) {
    $self->post_check_client_group($config,
 				  'client'.$mysqld->after('mysqld'),
 				  $mysqld->name())
@@ -333,12 +333,12 @@ sub post_check_embedded_group {
   my $mysqld= $config->group('mysqld') or
     croak "Can't run with embedded, config has no default mysqld section";
 
-  my $first_mysqld= $config->first_like('mysqld.') or
+  my $first_mysqld= $config->first_like('mysqld\.') or
     croak "Can't run with embedded, config has no mysqld";
 
   my %no_copy = map { $_ => 1 }
     (
-     '#log-error', # Embedded server writes stderr to mysqltest's log file
+     'log-error', # Embedded server writes stderr to mysqltest's log file
      'slave-net-timeout', # Embedded server are not build with replication
      'shared-memory-base-name', # No shared memory for embedded
     );
@@ -372,7 +372,7 @@ sub resolve_at_variable {
   }
   $res .= $after;
 
-  $config->insert($group->name(), $option->name(), $res)
+  $option->{value}= $res;
 }
 
 
@@ -457,7 +457,7 @@ sub new_config {
   }
 
   $self->run_section_rules($config,
-			   'mysqld.',
+			   'mysqld\.',
 			   @mysqld_rules);
 
   # [mysqlbinlog] need additional settings
