@@ -56,7 +56,7 @@ static int  first_packet_len;
 */
 char *srv_principal_name;
 char *srv_keytab_path;
-char *srv_mech_name=(char *)"";
+const char *srv_mech_name="";
 unsigned long srv_mech;
 
 /**
@@ -110,7 +110,7 @@ static int initialize_plugin(void *unused)
 
   strcpy(first_packet, srv_principal_name);
   strcpy(first_packet + strlen(srv_principal_name) + 1,srv_mech_name);
-  first_packet_len = strlen(srv_principal_name) + strlen(srv_mech_name) + 2;
+  first_packet_len = (int)(strlen(srv_principal_name) + strlen(srv_mech_name) + 2);
 
   return 0;
 }
