@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2019, MariaDB Corporation.
+Copyright (c) 2017, 2019, 2020 MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@ Created 5/30/1994 Heikki Tuuri
 #include "mem0mem.h"
 #include "dict0types.h"
 #include "btr0types.h"
+#include <vector>
 
 #include <ostream>
 
@@ -526,9 +527,6 @@ struct dtuple_t {
 	dfield_t*	fields;		/*!< fields */
 	ulint		n_v_fields;	/*!< number of virtual fields */
 	dfield_t*	v_fields;	/*!< fields on virtual column */
-	UT_LIST_NODE_T(dtuple_t) tuple_list;
-					/*!< data tuples can be linked into a
-					list using this field */
 #ifdef UNIV_DEBUG
 	ulint		magic_n;	/*!< magic number, used in
 					debug assertions */
