@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -27,8 +27,6 @@ Created 8/18/1994 Heikki Tuuri
 #ifndef ha0ha_h
 #define ha0ha_h
 
-#include "univ.i"
-
 #include "hash0hash.h"
 #include "page0types.h"
 #include "buf0types.h"
