@@ -101,7 +101,7 @@ os_mem_alloc_large(
 		my_atomic_addlint(
 			&os_total_large_mem_allocated, size);
 
-		UNIV_MEM_ALLOC(ptr, size);
+		MEM_UNDEFINED(ptr, size);
 		return(ptr);
 	}
 
@@ -125,7 +125,7 @@ skip:
 	} else {
 		my_atomic_addlint(
 			&os_total_large_mem_allocated, size);
-		UNIV_MEM_ALLOC(ptr, size);
+		MEM_UNDEFINED(ptr, size);
 	}
 #else
 	size = getpagesize();
@@ -141,7 +141,7 @@ skip:
 	} else {
 		my_atomic_addlint(
 			&os_total_large_mem_allocated, size);
-		UNIV_MEM_ALLOC(ptr, size);
+		MEM_UNDEFINED(ptr, size);
 	}
 #endif
 	return(ptr);
@@ -157,11 +157,13 @@ os_mem_free_large(
 {
 	ut_a(os_total_large_mem_allocated >= size);
 
+#ifdef __SANITIZE_ADDRESS__
 	// We could have manually poisoned that memory for ASAN.
 	// And we must unpoison it by ourself as specified in documentation
 	// for __asan_poison_memory_region() in sanitizer/asan_interface.h
 	// munmap() doesn't do it for us automatically.
-	UNIV_MEM_ALLOC(ptr, size);
+	MEM_MAKE_ADDRESSABLE(ptr, size);
+#endif /* __SANITIZE_ADDRESS__ */
 
 #ifdef HAVE_LINUX_LARGE_PAGES
 	if (my_use_large_pages && opt_large_page_size && !shmdt(ptr)) {
