@@ -8,8 +8,7 @@
 #include "rocksdb/slice.h"
 #include "utilities/cassandra/format.h"
 
-
-namespace rocksdb {
+namespace ROCKSDB_NAMESPACE {
 namespace cassandra {
 
 const char* CassandraCompactionFilter::Name() const {
@@ -17,13 +16,9 @@ const char* CassandraCompactionFilter::Name() const {
 }
 
 CompactionFilter::Decision CassandraCompactionFilter::FilterV2(
-  int level,
-  const Slice& key,
-  ValueType value_type,
-  const Slice& existing_value,
-  std::string* new_value,
-  std::string* skip_until) const {
-
+    int /*level*/, const Slice& /*key*/, ValueType value_type,
+    const Slice& existing_value, std::string* new_value,
+    std::string* /*skip_until*/) const {
   bool value_changed = false;
   RowValue row_value = RowValue::Deserialize(
     existing_value.data(), existing_value.size());
@@ -32,6 +27,10 @@ CompactionFilter::Decision CassandraCompactionFilter::FilterV2(
           ? row_value.RemoveExpiredColumns(&value_changed)
           : row_value.ConvertExpiredColumnsToTombstones(&value_changed);
 
+  if (value_type == ValueType::kValue) {
+    compacted = compacted.RemoveTombstones(gc_grace_period_in_seconds_);
+  }
+
   if(compacted.Empty()) {
     return Decision::kRemove;
   }
@@ -45,4 +44,4 @@ CompactionFilter::Decision CassandraCompactionFilter::FilterV2(
 }
 
 }  // namespace cassandra
-}  // namespace rocksdb
+}  // namespace ROCKSDB_NAMESPACE
