@@ -1,4 +1,4 @@
-/* Copyright (C) 2008-2014 Kentoku Shiba
+/* Copyright (C) 2008-2018 Kentoku Shiba
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 #define SPIDER_DB_WRAPPER_STR "mysql"
 #define SPIDER_DB_WRAPPER_LEN (sizeof(SPIDER_DB_WRAPPER_STR) - 1)
@@ -93,7 +93,7 @@
 #define SPIDER_SQL_LIMIT_LEN (sizeof(SPIDER_SQL_LIMIT_STR) - 1)
 #define SPIDER_SQL_INTO_STR "into "
 #define SPIDER_SQL_INTO_LEN (sizeof(SPIDER_SQL_INTO_STR) - 1)
-#define SPIDER_SQL_VALUES_STR ")values"
+#define SPIDER_SQL_VALUES_STR "values"
 #define SPIDER_SQL_VALUES_LEN (sizeof(SPIDER_SQL_VALUES_STR) - 1)
 #define SPIDER_SQL_SHARED_LOCK_STR " lock in share mode"
 #define SPIDER_SQL_SHARED_LOCK_LEN (sizeof(SPIDER_SQL_SHARED_LOCK_STR) - 1)
@@ -192,11 +192,32 @@
 #define SPIDER_SQL_PF_EQUAL_LEN (sizeof(SPIDER_SQL_PF_EQUAL_STR) - 1)
 #define SPIDER_SQL_GROUP_STR " group by "
 #define SPIDER_SQL_GROUP_LEN (sizeof(SPIDER_SQL_GROUP_STR) - 1)
+#define SPIDER_SQL_HAVING_STR " having "
+#define SPIDER_SQL_HAVING_LEN (sizeof(SPIDER_SQL_HAVING_STR) - 1)
 #define SPIDER_SQL_PLUS_STR " + "
 #define SPIDER_SQL_PLUS_LEN (sizeof(SPIDER_SQL_PLUS_STR) - 1)
 #define SPIDER_SQL_MINUS_STR " - "
 #define SPIDER_SQL_MINUS_LEN (sizeof(SPIDER_SQL_MINUS_STR) - 1)
 
+#define SPIDER_SQL_YEAR_STR "year"
+#define SPIDER_SQL_YEAR_LEN (sizeof(SPIDER_SQL_YEAR_STR) - 1)
+#define SPIDER_SQL_QUARTER_STR "quarter"
+#define SPIDER_SQL_QUARTER_LEN (sizeof(SPIDER_SQL_QUARTER_STR) - 1)
+#define SPIDER_SQL_MONTH_STR "month"
+#define SPIDER_SQL_MONTH_LEN (sizeof(SPIDER_SQL_MONTH_STR) - 1)
+#define SPIDER_SQL_WEEK_STR "week"
+#define SPIDER_SQL_WEEK_LEN (sizeof(SPIDER_SQL_WEEK_STR) - 1)
+#define SPIDER_SQL_DAY_STR "day"
+#define SPIDER_SQL_DAY_LEN (sizeof(SPIDER_SQL_DAY_STR) - 1)
+#define SPIDER_SQL_HOUR_STR "hour"
+#define SPIDER_SQL_HOUR_LEN (sizeof(SPIDER_SQL_HOUR_STR) - 1)
+#define SPIDER_SQL_MINUTE_STR "minute"
+#define SPIDER_SQL_MINUTE_LEN (sizeof(SPIDER_SQL_MINUTE_STR) - 1)
+#define SPIDER_SQL_SECOND_STR "second"
+#define SPIDER_SQL_SECOND_LEN (sizeof(SPIDER_SQL_SECOND_STR) - 1)
+#define SPIDER_SQL_MICROSECOND_STR "microsecond"
+#define SPIDER_SQL_MICROSECOND_LEN (sizeof(SPIDER_SQL_MICROSECOND_STR) - 1)
+
 #define SPIDER_SQL_SHOW_RECORDS_STR "select count(*) from "
 #define SPIDER_SQL_SHOW_RECORDS_LEN sizeof(SPIDER_SQL_SHOW_RECORDS_STR) - 1
 #define SPIDER_SQL_SHOW_INDEX_STR "show index from "
@@ -231,11 +252,19 @@
 #define SPIDER_SQL_B_STR "b"
 #define SPIDER_SQL_B_LEN (sizeof(SPIDER_SQL_B_STR) - 1)
 
+#define SPIDER_SQL_INDEX_IGNORE_STR " IGNORE INDEX "
+#define SPIDER_SQL_INDEX_IGNORE_LEN (sizeof(SPIDER_SQL_INDEX_IGNORE_STR) - 1)
+#define SPIDER_SQL_INDEX_USE_STR " USE INDEX "
+#define SPIDER_SQL_INDEX_USE_LEN (sizeof(SPIDER_SQL_INDEX_USE_STR) - 1)
+#define SPIDER_SQL_INDEX_FORCE_STR " FORCE INDEX "
+#define SPIDER_SQL_INDEX_FORCE_LEN (sizeof(SPIDER_SQL_INDEX_FORCE_STR) - 1)
+
 #define SPIDER_SQL_INT_LEN 20
 #define SPIDER_SQL_HANDLER_CID_LEN 6
 #define SPIDER_SQL_HANDLER_CID_FORMAT "t%05u"
-#define SPIDER_UDF_PING_TABLE_PING_ONLY (1 << 0)
-#define SPIDER_UDF_PING_TABLE_USE_WHERE (1 << 1)
+#define SPIDER_UDF_PING_TABLE_PING_ONLY                (1 << 0)
+#define SPIDER_UDF_PING_TABLE_USE_WHERE                (1 << 1)
+#define SPIDER_UDF_PING_TABLE_USE_ALL_MONITORING_NODES (1 << 2)
 
 int spider_db_connect(
   const SPIDER_SHARE *share,
@@ -243,6 +272,13 @@ int spider_db_connect(
   int link_idx
 );
 
+int spider_db_ping_internal(
+  SPIDER_SHARE *share,
+  SPIDER_CONN *conn,
+  int all_link_idx,
+  int *need_mon
+);
+
 int spider_db_ping(
   ha_spider *spider,
   SPIDER_CONN *conn,
@@ -305,7 +341,7 @@ int spider_db_query_for_bulk_update(
   ha_spider *spider,
   SPIDER_CONN *conn,
   int link_idx,
-  uint *dup_key_found
+  ha_rows *dup_key_found
 );
 
 size_t spider_db_real_escape_string(
@@ -376,7 +412,20 @@ int spider_db_unlock_tables(
 
 int spider_db_append_name_with_quote_str(
   spider_string *str,
-  char *name,
+  const char *name,
+  uint dbton_id
+);
+
+int spider_db_append_name_with_quote_str(
+  spider_string *str,
+  LEX_CSTRING &name,
+  uint dbton_id
+);
+
+int spider_db_append_name_with_quote_str_internal(
+  spider_string *str,
+  const char *name,
+  int length,
   uint dbton_id
 );
 
@@ -530,6 +579,10 @@ void spider_db_free_one_result(
   SPIDER_RESULT *result
 );
 
+void spider_db_free_one_quick_result(
+  SPIDER_RESULT *result
+);
+
 int spider_db_free_result(
   ha_spider *spider,
   bool final
@@ -693,7 +746,7 @@ int spider_db_bulk_update_size_limit(
 
 int spider_db_bulk_update_end(
   ha_spider *spider,
-  uint *dup_key_found
+  ha_rows *dup_key_found
 );
 
 int spider_db_bulk_update(
@@ -709,19 +762,27 @@ int spider_db_update(
 );
 
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
 int spider_db_direct_update(
   ha_spider *spider,
   TABLE *table,
   KEY_MULTI_RANGE *ranges,
   uint range_count,
-  uint *update_rows
+  ha_rows *update_rows
 );
+#else
+int spider_db_direct_update(
+  ha_spider *spider,
+  TABLE *table,
+  ha_rows *update_rows
+);
+#endif
 #endif
 
 #ifdef HA_CAN_BULK_ACCESS
 int spider_db_bulk_direct_update(
   ha_spider *spider,
-  uint *update_rows
+  ha_rows *update_rows
 );
 #endif
 
@@ -738,13 +799,21 @@ int spider_db_delete(
 );
 
 #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
 int spider_db_direct_delete(
   ha_spider *spider,
   TABLE *table,
   KEY_MULTI_RANGE *ranges,
   uint range_count,
-  uint *delete_rows
+  ha_rows *delete_rows
 );
+#else
+int spider_db_direct_delete(
+  ha_spider *spider,
+  TABLE *table,
+  ha_rows *delete_rows
+);
+#endif
 #endif
 
 int spider_db_delete_all_rows(
@@ -786,13 +855,31 @@ int spider_db_flush_logs(
   ha_spider *spider
 );
 
+Field *spider_db_find_field_in_item_list(
+  Item **item_list,
+  uint item_count,
+  uint start_item,
+  spider_string *str,
+  const char *func_name,
+  int func_name_length
+);
+
 int spider_db_print_item_type(
   Item *item,
+  Field *field,
   ha_spider *spider,
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
+);
+
+int spider_db_print_item_type_default(
+  Item *item,
+  ha_spider *spider,
+  spider_string *str
 );
 
 int spider_db_open_item_cond(
@@ -801,7 +888,9 @@ int spider_db_open_item_cond(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_func(
@@ -810,7 +899,9 @@ int spider_db_open_item_func(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 #ifdef HANDLER_HAS_DIRECT_AGGREGATE
@@ -820,7 +911,9 @@ int spider_db_open_item_sum_func(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 #endif
 
@@ -830,7 +923,9 @@ int spider_db_open_item_ident(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_field(
@@ -839,7 +934,9 @@ int spider_db_open_item_field(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_ref(
@@ -848,7 +945,9 @@ int spider_db_open_item_ref(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_row(
@@ -857,34 +956,57 @@ int spider_db_open_item_row(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_string(
   Item *item,
+  Field *field,
   ha_spider *spider,
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_int(
   Item *item,
+  Field *field,
   ha_spider *spider,
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_open_item_cache(
   Item_cache *item_cache,
+  Field *field,
   ha_spider *spider,
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
+);
+
+int spider_db_open_item_insert_value(
+  Item_insert_value *item_insert_value,
+  Field *field,
+  ha_spider *spider,
+  spider_string *str,
+  const char *alias,
+  uint alias_length,
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 
 int spider_db_append_condition(
@@ -900,7 +1022,9 @@ int spider_db_append_update_columns(
   spider_string *str,
   const char *alias,
   uint alias_length,
-  uint dbton_id
+  uint dbton_id,
+  bool use_fields,
+  spider_fields *fields
 );
 #endif
 
